@@ -10,6 +10,7 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/CORSMode.h"
 #include "mozilla/Mutex.h"
+#include "mozilla/EnumSet.h"
 #include "mozilla/UniquePtr.h"
 
 #include "imgILoader.h"
@@ -17,6 +18,7 @@
 #include "nsWeakReference.h"
 #include "nsIContentSniffer.h"
 #include "nsRefPtrHashtable.h"
+#include "nsTHashSet.h"
 #include "nsExpirationTracker.h"
 #include "ImageCacheKey.h"
 #include "imgRequest.h"
@@ -40,30 +42,10 @@ class Document;
 
 class imgCacheEntry {
  public:
+  NS_INLINE_DECL_REFCOUNTING(imgCacheEntry)
+
   imgCacheEntry(imgLoader* loader, imgRequest* request,
                 bool aForcePrincipalCheck);
-  ~imgCacheEntry();
-
-  nsrefcnt AddRef() {
-    MOZ_ASSERT(int32_t(mRefCnt) >= 0, "illegal refcnt");
-    NS_ASSERT_OWNINGTHREAD(imgCacheEntry);
-    ++mRefCnt;
-    NS_LOG_ADDREF(this, mRefCnt, "imgCacheEntry", sizeof(*this));
-    return mRefCnt;
-  }
-
-  nsrefcnt Release() {
-    MOZ_ASSERT(0 != mRefCnt, "dup release");
-    NS_ASSERT_OWNINGTHREAD(imgCacheEntry);
-    --mRefCnt;
-    NS_LOG_RELEASE(this, mRefCnt, "imgCacheEntry");
-    if (mRefCnt == 0) {
-      mRefCnt = 1; /* stabilize */
-      delete this;
-      return 0;
-    }
-    return mRefCnt;
-  }
 
   uint32_t GetDataSize() const { return mDataSize; }
   void SetDataSize(uint32_t aDataSize) {
@@ -82,8 +64,8 @@ class imgCacheEntry {
 
   void UpdateLoadTime();
 
-  int32_t GetExpiryTime() const { return mExpiryTime; }
-  void SetExpiryTime(int32_t aExpiryTime) {
+  uint32_t GetExpiryTime() const { return mExpiryTime; }
+  void SetExpiryTime(uint32_t aExpiryTime) {
     mExpiryTime = aExpiryTime;
     Touch();
   }
@@ -107,6 +89,12 @@ class imgCacheEntry {
 
   bool ForcePrincipalCheck() const { return mForcePrincipalCheck; }
 
+  bool HasNotified() const { return mHasNotified; }
+  void SetHasNotified() {
+    MOZ_ASSERT(!mHasNotified);
+    mHasNotified = true;
+  }
+
   imgLoader* Loader() const { return mLoader; }
 
  private:  // methods
@@ -119,22 +107,21 @@ class imgCacheEntry {
 
   // Private, unimplemented copy constructor.
   imgCacheEntry(const imgCacheEntry&);
+  ~imgCacheEntry();
 
  private:  // data
-  nsAutoRefCnt mRefCnt;
-  NS_DECL_OWNINGTHREAD
-
   imgLoader* mLoader;
   RefPtr<imgRequest> mRequest;
   uint32_t mDataSize;
   int32_t mTouchedTime;
   uint32_t mLoadTime;
-  int32_t mExpiryTime;
+  uint32_t mExpiryTime;
   nsExpirationState mExpirationState;
   bool mMustValidate : 1;
   bool mEvicted : 1;
   bool mHasNoProxies : 1;
   bool mForcePrincipalCheck : 1;
+  bool mHasNotified : 1;
 };
 
 #include <vector>
@@ -187,11 +174,11 @@ class imgLoader final : public imgILoader,
   virtual ~imgLoader();
 
  public:
-  typedef mozilla::image::ImageCacheKey ImageCacheKey;
-  typedef nsRefPtrHashtable<nsGenericHashKey<ImageCacheKey>, imgCacheEntry>
-      imgCacheTable;
-  typedef nsTHashtable<nsPtrHashKey<imgRequest>> imgSet;
-  typedef mozilla::Mutex Mutex;
+  using ImageCacheKey = mozilla::image::ImageCacheKey;
+  using imgCacheTable =
+      nsRefPtrHashtable<nsGenericHashKey<ImageCacheKey>, imgCacheEntry>;
+  using imgSet = nsTHashSet<imgRequest*>;
+  using Mutex = mozilla::Mutex;
 
   NS_DECL_ISUPPORTS
   NS_DECL_IMGILOADER
@@ -240,6 +227,9 @@ class imgLoader final : public imgILoader,
   imgLoader();
   nsresult Init();
 
+  bool IsImageAvailable(nsIURI*, nsIPrincipal* aTriggeringPrincipal,
+                        mozilla::CORSMode, mozilla::dom::Document*);
+
   [[nodiscard]] nsresult LoadImage(
       nsIURI* aURI, nsIURI* aInitialDocumentURI, nsIReferrerInfo* aReferrerInfo,
       nsIPrincipal* aLoadingPrincipal, uint64_t aRequestContextID,
@@ -248,7 +238,7 @@ class imgLoader final : public imgILoader,
       nsLoadFlags aLoadFlags, nsISupports* aCacheKey,
       nsContentPolicyType aContentPolicyType, const nsAString& initiatorType,
       bool aUseUrgentStartForChannel, bool aLinkPreload,
-      imgRequestProxy** _retval);
+      uint64_t aEarlyHintPreloaderId, imgRequestProxy** _retval);
 
   [[nodiscard]] nsresult LoadImageWithChannel(
       nsIChannel* channel, imgINotificationObserver* aObserver,
@@ -272,16 +262,19 @@ class imgLoader final : public imgILoader,
    * @param aAcceptedMimeTypes Which kinds of MIME types to treat as images.
    */
   static bool SupportImageWithMimeType(
-      const char* aMimeType,
-      AcceptedMimeTypes aAccept = AcceptedMimeTypes::IMAGES);
+      const nsACString&, AcceptedMimeTypes aAccept = AcceptedMimeTypes::IMAGES);
 
   static void GlobalInit();  // for use by the factory
   static void Shutdown();    // for use by the factory
   static void ShutdownMemoryReporter();
 
-  nsresult ClearChromeImageCache();
-  nsresult ClearImageCache();
-  void MinimizeCaches();
+  enum class ClearOption {
+    ChromeOnly,
+    UnusedOnly,
+  };
+  using ClearOptions = mozilla::EnumSet<ClearOption>;
+  nsresult ClearImageCache(ClearOptions = {});
+  void MinimizeCache() { ClearImageCache({ClearOption::UnusedOnly}); }
 
   nsresult InitCache();
 
@@ -326,6 +319,9 @@ class imgLoader final : public imgILoader,
 
   void VerifyCacheSizes();
 
+  nsresult RemoveEntriesInternal(nsIPrincipal* aPrincipal,
+                                 const nsACString* aBaseDomain);
+
   // The image loader maintains a hash table of all imgCacheEntries. However,
   // only some of them will be evicted from the cache: those who have no
   // imgRequestProxies watching their imgRequests.
@@ -340,23 +336,20 @@ class imgLoader final : public imgILoader,
   bool SetHasNoProxies(imgRequest* aRequest, imgCacheEntry* aEntry);
   bool SetHasProxies(imgRequest* aRequest);
 
-  // This method converts imgIRequest::CORS_* values to mozilla::CORSMode
-  // values.
-  static mozilla::CORSMode ConvertToCORSMode(uint32_t aImgCORS);
-
  private:  // methods
   static already_AddRefed<imgLoader> CreateImageLoader();
 
-  bool PreferLoadFromCache(nsIURI* aURI) const;
-
-  bool ValidateEntry(
-      imgCacheEntry* aEntry, nsIURI* aURI, nsIURI* aInitialDocumentURI,
-      nsIReferrerInfo* aReferrerInfo, nsILoadGroup* aLoadGroup,
-      imgINotificationObserver* aObserver,
-      mozilla::dom::Document* aLoadingDocument, nsLoadFlags aLoadFlags,
-      nsContentPolicyType aLoadPolicyType, bool aCanMakeNewChannel,
-      bool* aNewChannelCreated, imgRequestProxy** aProxyRequest,
-      nsIPrincipal* aTriggeringPrincipal, int32_t aCORSMode, bool aLinkPreload);
+  bool ValidateEntry(imgCacheEntry* aEntry, nsIURI* aURI,
+                     nsIURI* aInitialDocumentURI,
+                     nsIReferrerInfo* aReferrerInfo, nsILoadGroup* aLoadGroup,
+                     imgINotificationObserver* aObserver,
+                     mozilla::dom::Document* aLoadingDocument,
+                     nsLoadFlags aLoadFlags,
+                     nsContentPolicyType aLoadPolicyType,
+                     bool aCanMakeNewChannel, bool* aNewChannelCreated,
+                     imgRequestProxy** aProxyRequest,
+                     nsIPrincipal* aTriggeringPrincipal, mozilla::CORSMode,
+                     bool aLinkPreload, uint64_t aEarlyHintPreloaderId);
 
   bool ValidateRequestWithNewChannel(
       imgRequest* request, nsIURI* aURI, nsIURI* aInitialDocumentURI,
@@ -365,8 +358,16 @@ class imgLoader final : public imgILoader,
       mozilla::dom::Document* aLoadingDocument, uint64_t aInnerWindowId,
       nsLoadFlags aLoadFlags, nsContentPolicyType aContentPolicyType,
       imgRequestProxy** aProxyRequest, nsIPrincipal* aLoadingPrincipal,
-      int32_t aCORSMode, bool aLinkPreload, bool* aNewChannelCreated);
+      mozilla::CORSMode, bool aLinkPreload, uint64_t aEarlyHintPreloaderId,
+      bool* aNewChannelCreated);
 
+  void NotifyObserversForCachedImage(imgCacheEntry* aEntry, imgRequest* request,
+                                     nsIURI* aURI,
+                                     nsIReferrerInfo* aReferrerInfo,
+                                     mozilla::dom::Document* aLoadingDocument,
+                                     nsIPrincipal* aLoadingPrincipal,
+                                     mozilla::CORSMode,
+                                     uint64_t aEarlyHintPreloaderId);
   // aURI may be different from imgRequest's URI in the case of blob URIs, as we
   // can share requests with different URIs.
   nsresult CreateNewProxyForRequest(imgRequest* aRequest, nsIURI* aURI,
@@ -376,15 +377,10 @@ class imgLoader final : public imgILoader,
                                     nsLoadFlags aLoadFlags,
                                     imgRequestProxy** _retval);
 
-  nsresult EvictEntries(imgCacheTable& aCacheToClear);
-  nsresult EvictEntries(imgCacheQueue& aQueueToClear);
+  nsresult EvictEntries(bool aChromeOnly);
 
-  imgCacheTable& GetCache(bool aForChrome);
-  imgCacheTable& GetCache(const ImageCacheKey& aKey);
-  imgCacheQueue& GetCacheQueue(bool aForChrome);
-  imgCacheQueue& GetCacheQueue(const ImageCacheKey& aKey);
-  void CacheEntriesChanged(bool aForChrome, int32_t aSizeDiff = 0);
-  void CheckCacheLimits(imgCacheTable& cache, imgCacheQueue& queue);
+  void CacheEntriesChanged(int32_t aSizeDiff);
+  void CheckCacheLimits();
 
  private:  // data
   friend class imgCacheEntry;
@@ -393,14 +389,11 @@ class imgLoader final : public imgILoader,
   imgCacheTable mCache;
   imgCacheQueue mCacheQueue;
 
-  imgCacheTable mChromeCache;
-  imgCacheQueue mChromeCacheQueue;
-
   // Hash set of every imgRequest for this loader that isn't in mCache or
   // mChromeCache. The union over all imgLoader's of mCache, mChromeCache, and
   // mUncachedImages should be every imgRequest that is alive. These are weak
   // pointers so we rely on the imgRequest destructor to remove itself.
-  imgSet mUncachedImages;
+  imgSet mUncachedImages MOZ_GUARDED_BY(mUncachedImagesMutex);
   // The imgRequest can have refs to them held on non-main thread, so we need
   // a mutex because we modify the uncached images set from the imgRequest
   // destructor.
